==> opendal-0.34.0.crate: opendal-0.34.0/src/services/sftp/writer.rs <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

use async_trait::async_trait;
use bb8::PooledConnection;
use bytes::Bytes;

use super::backend::Manager;
use crate::raw::oio;
use crate::{Error, ErrorKind, Result};

pub struct SftpWriter {
    conn: PooledConnection<'static, Manager>,
    path: String,
}

impl SftpWriter {
    pub fn new(conn: PooledConnection<'static, Manager>, path: String) -> Self {
        SftpWriter { conn, path }
    }
}

#[async_trait]
impl oio::Write for SftpWriter {
    async fn write(&mut self, bs: Bytes) -> Result<()> {
        let mut file = self.conn.sftp.create(&self.path).await?;

        file.write_all(&bs).await?;

        Ok(())
    }

    async fn abort(&mut self) -> Result<()> {
        Err(Error::new(
            ErrorKind::Unsupported,
            "SFTP does not support aborting writes",
        ))
    }

    async fn close(&mut self) -> Result<()> {
        Ok(())
    }
}

==> opendal-0.37.0.crate: opendal-0.37.0/src/services/sftp/writer.rs <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

use async_trait::async_trait;
use bytes::Bytes;
use openssh_sftp_client::file::File;

use crate::raw::oio;
use crate::Error;
use crate::ErrorKind;
use crate::Result;

pub struct SftpWriter {
    file: File,
}

impl SftpWriter {
    pub fn new(file: File) -> Self {
        SftpWriter { file }
    }
}

#[async_trait]
impl oio::Write for SftpWriter {
    async fn write(&mut self, bs: Bytes) -> Result<()> {
        self.file.write_all(&bs).await?;

        Ok(())
    }

    async fn abort(&mut self) -> Result<()> {
        Err(Error::new(
            ErrorKind::Unsupported,
            "SFTP does not support aborting writes",
        ))
    }

    async fn close(&mut self) -> Result<()> {
        Ok(())
    }
}

#[async_trait]
impl oio::Append for SftpWriter {
    async fn append(&mut self, bs: Bytes) -> Result<()> {
        self.file.write_all(&bs).await?;

        Ok(())
    }

    async fn close(&mut self) -> Result<()> {
        Ok(())
    }
}
